!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              DEFINE_TR_CLOUD    Jan. 2014                                    |
!                                                                              |
!              based on define_cloud                                           |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine define_trcloud (trcl,params) !might use threadinfo

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! if irestart<tr_activation, this routine allocates the tracking cloud arrays.
! Otherwise it reads from a user supplied file name the tracking cloud as it
! was at the end of a previous run. In this case, since the run output 
! files contain all the octree+lsf+icloud+surface informations, the routine 
! first reads dummy parameters until it gets to the real interesting 
! cloud information.

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use definitions
!use mpi
!use threads

implicit none

include 'mpif.h'

type (tracking_cloud) trcl
type (parameters) params

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|
logical isostasyout,isobcout,nestout,compactionout,ptrackingout
integer noctree,nnode,nleaves,nface,nlsf,clnp
integer trgridshape,gridstep,ntrpx,ntrpy,ntrpz
double precision current_time,dx,dy,dz

integer err,iproc,nproc,ierr,ilsf,i,j,k

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)


!------------------------------------------------------------------------------|
! Allocating the tracking and surfaced cloud and filling with default values   |
!------------------------------------------------------------------------------|

!allocate tracking cloud
trcl%np=0
trcl%ntracks=0
allocate (trcl%x(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%x in define_trcloud$')
allocate (trcl%y(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%y in define_trcloud$')
allocate (trcl%z(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%z in define_trcloud$')
allocate (trcl%x0(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%x0 in define_trcloud$')
allocate (trcl%y0(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%y0 in define_trcloud$')
allocate (trcl%z0(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%z0 in define_trcloud$')
allocate (trcl%press(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%press in define_trcloud$')
allocate (trcl%temp(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%temp in define_trcloud$')
allocate (trcl%lsf0(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%lsf0 in define_trcloud$')
allocate (trcl%trackstart(params%ntrp), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%trackstart in define_trcloud$')
allocate (trcl%tracks(1,3), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%tracks in define_trcloud$')


!filling arrays with default values
trcl%x = -1.d0
trcl%y = -1.d0
trcl%z = -1.d0
trcl%x0 = -1.d0
trcl%y0 = -1.d0
trcl%z0 = -1.d0
trcl%press = 0.d0
trcl%temp = -1.d0
trcl%lsf0 = -2.d0 !lsf's range from -1 to 1, 0 at the surface
trcl%trackstart=params%tr_activation
trcl%tracks=0


!------------------------------------------------------------------------------|
! Reading from restart file                                                    |
!------------------------------------------------------------------------------|

!If restart file has particle tracking information, read and load
!This porcedure reads dummy parameters until it comes to the relevant information
!at the end of the file.
!If there is no restart information, create cloud particles

load: if (params%irestart > 0 .AND. params%irestart>=params%tr_activation) then
    open (19,file=trim(params%restartfile),status='old',form='unformatted')
    ! Read version number
    read (19) 
   
    ! Read optional output flags
    read (19) isostasyout,isobcout,nestout,compactionout,ptrackingout
   
    if (.not. ptrackingout) then
        call stop_run ('Error: No tracking cloud information in restart file to read from!')
    end if
   
    !Read array lengths
    read (19) noctree,nnode,nleaves,nface,nlsf,clnp,current_time 
   
    ! Dummy reads
    ! info on octree solve nodes
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    ! Isostasy and compaction output
    if (isostasyout) read (19)
    if (compactionout) read (19)
    !icon array
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    !compaction output
    if (compactionout) read (19)
    ! octree information
    read (19)
    ! bad face information
    read (19)
    ! void information
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    ! bad faces information
    read (19)
    ! surface information
    do ilsf=1,nlsf                                  
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
       read (19)
    enddo
    
    !Cloud Information
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
    read (19)
   
    ! read isostasy basal displacement array (optional) - dwhipp 07.13
    if (isobcout) then
      read (19)
      read (19)
    endif
   
    ! read nest info (optional) - dwhipp 07.13
    if (nestout) read(19)
   
    ! read compaction density strings (optional) - dwhipp 07.13
    if (compactionout) then
      read (19)
      do j=1,2**params%levelmax_oct
        do i=1,2**params%levelmax_oct
          read (19)
          read (19) 
        enddo
      enddo
    endif
   
    !Reading tracking particle information
    readtrcl: if (ptrackingout) then
        read (19) trcl%np, trcl%ntracks
        
        !read tracking cloud up to current time step
        read(19) trcl%x
        read(19) trcl%y
        read(19) trcl%z
        ! correct for vertical exaggeration
        trcl%z=trcl%z/params%vex
        !read(19) trcl%lsf0
        read(19) trcl%trackstart
   
        !read track infomation
        deallocate (trcl%tracks)
        allocate (trcl%tracks(trcl%ntracks,3), stat=err) ; if (err.ne.0) call stop_run ('Error alloc trcl%tracks in define_trcloud$')
        do i = 1,trcl%ntracks
            read(19) trcl%tracks(i,:)
        end do
   
    end if readtrcl
   
    close (19)

end if load 

return
end

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
